"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.guard = void 0;
const symbols_1 = require("./symbols");
const utils_1 = require("./utils");
function guard(name, fn, 
// The difference between Options and ExtraOptions is that for Options, the guard implementation also provides the implementation of the options
// Scheams using the guard then only provide a value to the guard whereas for ExtraOptions, both the guard and the caller may implement functions which are then both considered
// Also, we purposefully don't initialize it by default to cut some corners further down when checking as we can just check if handleOptions === undefined
handleOptions, defaultOptions = {}) {
    function getFnWithErrors(options) {
        return (value, path = []) => {
            var _a, _b, _c, _d, _e;
            const data = fn(value, options);
            if (!(0, utils_1.isValid)(data) || !((_a = handleOptions === null || handleOptions === void 0 ? void 0 : handleOptions.validate) !== null && _a !== void 0 ? _a : utils_1.trueFn)(data)) {
                return {
                    valid: false,
                    data: undefined,
                    errors: [
                        {
                            message: `vality.${name}.base`,
                            path,
                            options,
                            value,
                        },
                    ],
                };
            }
            if (!((_b = options.validate) !== null && _b !== void 0 ? _b : utils_1.trueFn)(data)) {
                return {
                    valid: false,
                    data: undefined,
                    errors: [
                        {
                            message: `vality.${name}.custom`,
                            path,
                            options,
                            value,
                        },
                    ],
                };
            }
            const transformedData = ((_c = handleOptions === null || handleOptions === void 0 ? void 0 : handleOptions.transform) !== null && _c !== void 0 ? _c : utils_1.identity)(data);
            if (handleOptions === undefined)
                return { valid: true, data: ((_d = options.transform) !== null && _d !== void 0 ? _d : utils_1.identity)(transformedData), errors: [] };
            const keysWithError = Object.keys(options).filter(k => k !== "transform" &&
                k !== "validate" &&
                handleOptions[k] !== undefined &&
                !handleOptions[k](data, options[k], options));
            if (keysWithError.length === 0) {
                return {
                    valid: true,
                    data: ((_e = options.transform) !== null && _e !== void 0 ? _e : utils_1.identity)(transformedData),
                    errors: [],
                };
            }
            return {
                valid: false,
                data: undefined,
                errors: keysWithError.map(k => ({
                    message: `vality.${name}.options.${k}`,
                    options,
                    path,
                    value,
                })),
            };
        };
    }
    return Object.assign((options) => {
        return {
            [symbols_1._validate]: getFnWithErrors(Object.assign(Object.assign({}, defaultOptions), options)),
            [symbols_1._type]: undefined,
        };
    }, {
        [symbols_1._validate]: getFnWithErrors(defaultOptions),
        [symbols_1._type]: undefined,
    });
}
exports.guard = guard;
